-- Rymdskepp

function setup()
    viewer.mode = FULLSCREEN
    setupMenu()
end

function setupMenu()
    noStroke()
    rectMode(CENTER)
    textMode(CENTER)
    fontSize(40)
    local menu = ecs.System("label")
    function menu:update(e)
        if e.touched then
            fill(233, 207, 80) 
        else
            fill(130)
        end
        
        rect(e.pos.x, e.pos.y, 250,60)
        fill(224)
        text(e.label, e.pos.x, e.pos.y)
    end
    function menu:touched(e, touch)
        if e .pos:dist(touch.pos) < 40 then
            if touch.state ~= ENDED then
                e.touched = true
            else
                setupGame(e.isHost)
            end
        else
            e.touched = false
        end
    end
    world = ecs.World(menu)
    world:add(
    {label="Host game", pos=vec2(WIDTH/2, HEIGHT/2+80), isHost=true}, 
    {label="Connect", pos=vec2(WIDTH/2, HEIGHT/2), isHost=false}
    )
end

function setupGame(isHost)
    local bounce = ecs.System("pos and vel")
    function bounce:update(e, dt)
        e.pos = e.pos + e.vel * dt
        if e.pos.x < 0 or e.pos.x > WIDTH then
            e.vel.x = -e.vel.x
        end
        if e.pos.y < 100 or e.pos.y > (HEIGHT-100) then
            e.vel.y = -e.vel.y
        end
    end
    
    local drawPixels = ecs.System("pos and fill")
    function drawPixels:update(e, dt)
        fill(255, 20, 0)
        rect(e.pos.x, e.pos.y, 8, 8)
    end
    
    local drawSprites = ecs.System("pos and (not fill)")
    function drawSprites:update(e, dt)
        sprite(asset.builtin.Space_Art.Green_Ship, e.pos.x, e.pos.y, 40, 40)
    end
    
    local fadeSystem = ecs.System("fill")
    function fadeSystem:update(e, dt)
        e.fill = e.fill:mix(color(0, 0), 1-dt*.5)
        if e.fill.a <= 150 then
            -- delete entity when faded
            return ecs.DELETE
        end
    end
    
    local score = {player1=0, player2=0}
    local explode = ecs.System("pos and (not fill)")
    function explode:touched(e, touch)
        if touch.state == BEGAN and e.pos:dist(touch.pos) < 19 then
            createExplosion(e.pos, 20)
            if touch.isHost then
                score.player1 = score.player1+1
            else
                score.player2 = score.player2+1
            end
            return ecs.DELETE
        end
    end
    
    local scoreSys = ecs.System("player1")
    function scoreSys:update(e)
        fill(151, 80, 233)
        fontSize(30)
        text("P1:"..e.player1.." P2:"..e.player2, WIDTH/2,HEIGHT-60)
    end
    
    world = ecs.World(bounce, fadeSystem, drawPixels, drawSprites, explode, scoreSys)
    
    if isHost then
        world:add(score)
        -- add 10 random sprites
        for i = 1, 10 do
            world:add({
                pos=vec2(WIDTH/2, HEIGHT/2),
                vel=vec2(math.random(150,300),0):rotate(i*10),
            })
        end
    end
    world:enableNetwork(isHost)
end


function draw()
    background(40, 40, 50)
    world:update(DeltaTime)
end

function touched(touch)
    world:touched(touch)
end

function createExplosion(pos, n)
    for i = 1,n do
        world:add({
            pos=pos,
            vel=vec2(math.random(-100,100),math.random(-150,150)),
            fill=color(255, 176, 0)
        })
    end
end