-- UDP network class
socket=require("socket")

Net = class()

function Net:init(port)
    self.port = port
    self:get_ip()
    self.server=socket.udp()
    self.server:setsockname(self.ip, port)
    self.hostname = socket.dns.gethostname() 
    print("ip: " .. self.ip)
    print("hostname: " .. self.hostname)
    self.server:settimeout(0)
    self.client=socket.udp()
    self.client:settimeout(0)
    self.listeners = {}
end

function Net:update(isHost, ...)
    local data, remote_ip = self:receive()
    if data ~= nil then
        if data.action == "connect" then
            table.insert(self.listeners, remote_ip)
            data = nil
        end
    end
    for msg in ipairs(...) do
        local data = msgpack.encode(msg)
        for ip in ipairs(self.listeners) do
            self.client:setpeername(ip, self.port)
            self.client:send(data)
        end
    end
    return data
end


function Net:get_ip()
    local server=socket.udp()
    server:setpeername("1.1.1.1",self.port)
    self.ip,_=server:getsockname()
    self.networkid, self.hostid=string.match(self.ip,"(%d+.%d+.%d+.)(%d+)")
end

function Net:broadcast(data)
    data = msgpack.encode(data)
    for hostid=1,254 do
        if hostid~=tonumber(self.hostid) then
            self.client:setpeername(self.networkid..hostid,self.port)
            self.client:send(data)
        end
    end
end

function Net:send(ip, data)
    data = msgpack.encode(data)
    self.client:setpeername(ip, self.port)
    self.client:send(data)
end

function Net:receive()
    local data, remoteIp, port=self.server:receivefrom()
    if data~=nil then
        data = msgpack.decode(data)
        return data, remoteIp
    else
        return nil, nil
    end
end
